@@ -39,6 +39,11 @@ class ComputeCloudMixin(_normalize.Normalizer):
         self._servers_time = 0
         self._servers_lock = threading.Lock()
 
+    @property
+    def _compute_region(self):
+        # This is only used in exception messages. Can we get rid of it?
+        return self.config.get_region_name('compute')
+
     def get_flavor_name(self, flavor_id):
         flavor = self.get_flavor(flavor_id, get_extra=False)
         if flavor:
@@ -80,20 +85,15 @@ class ComputeCloudMixin(_normalize.Normalizer):
         return extension_name in self._nova_extensions()
 
     def search_keypairs(self, name_or_id=None, filters=None):
-        keypairs = self.list_keypairs()
+        keypairs = self.list_keypairs(
+            filters=filters if isinstance(filters, dict) else None
+        )
         return _utils._filter_list(keypairs, name_or_id, filters)
 
     def search_flavors(self, name_or_id=None, filters=None, get_extra=True):
         flavors = self.list_flavors(get_extra=get_extra)
         return _utils._filter_list(flavors, name_or_id, filters)
 
-    def search_security_groups(self, name_or_id=None, filters=None):
-        # `filters` could be a dict or a jmespath (str)
-        groups = self.list_security_groups(
-            filters=filters if isinstance(filters, dict) else None
-        )
-        return _utils._filter_list(groups, name_or_id, filters)
-
     def search_servers(
             self, name_or_id=None, filters=None, detailed=False,
             all_projects=False, bare=False):
@@ -115,17 +115,16 @@ class ComputeCloudMixin(_normalize.Normalizer):
         server_groups = self.list_server_groups()
         return _utils._filter_list(server_groups, name_or_id, filters)
 
-    def list_keypairs(self):
+    def list_keypairs(self, filters=None):
         """List all available keypairs.
 
         :returns: A list of ``munch.Munch`` containing keypair info.
 
         """
-        data = proxy._json_response(
-            self.compute.get('/os-keypairs'),
-            error_message="Error fetching keypair list")
-        return self._normalize_keypairs([
-            k['keypair'] for k in self._get_and_munchify('keypairs', data)])
+        if not filters:
+            filters = {}
+        return list(self.compute.keypairs(allow_unknown_params=True,
+                                          **filters))
 
     @_utils.cache_on_arguments()
     def list_availability_zone_names(self, unavailable=False):
@@ -138,19 +137,17 @@ class ComputeCloudMixin(_normalize.Normalizer):
                   list could not be fetched.
         """
         try:
-            data = proxy._json_response(
-                self.compute.get('/os-availability-zone'))
-        except exc.OpenStackCloudHTTPError:
+            zones = self.compute.availability_zones()
+            ret = []
+            for zone in zones:
+                if zone.state['available'] or unavailable:
+                    ret.append(zone.name)
+            return ret
+        except exceptions.SDKException:
             self.log.debug(
                 "Availability zone list could not be fetched",
                 exc_info=True)
             return []
-        zones = self._get_and_munchify('availabilityZoneInfo', data)
-        ret = []
-        for zone in zones:
-            if zone['zoneState']['available'] or unavailable:
-                ret.append(zone['zoneName'])
-        return ret
 
     @_utils.cache_on_arguments()
     def list_flavors(self, get_extra=False):
@@ -198,12 +195,11 @@ class ComputeCloudMixin(_normalize.Normalizer):
         if not self._has_secgroups():
             return []
 
-        data = proxy._json_response(
-            self.compute.get(
-                '/servers/{server_id}/os-security-groups'.format(
-                    server_id=server['id'])))
-        return self._normalize_secgroups(
-            self._get_and_munchify('security_groups', data))
+        server = self.compute.get_server(server)
+
+        server.fetch_security_groups(self.compute)
+
+        return self._normalize_secgroups(server.security_groups)
 
     def _get_server_security_groups(self, server, security_groups):
         if not self._has_secgroups():
@@ -255,9 +251,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             return False
 
         for sg in security_groups:
-            proxy._json_response(self.compute.post(
-                '/servers/%s/action' % server['id'],
-                json={'addSecurityGroup': {'name': sg.name}}))
+            self.compute.add_security_group_to_server(server, sg)
 
         return True
 
@@ -283,11 +277,9 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         for sg in security_groups:
             try:
-                proxy._json_response(self.compute.post(
-                    '/servers/%s/action' % server['id'],
-                    json={'removeSecurityGroup': {'name': sg.name}}))
+                self.compute.remove_security_group_from_server(server, sg)
 
-            except exc.OpenStackCloudURINotFound:
+            except exceptions.ResourceNotFound:
                 # NOTE(jamielennox): Is this ok? If we remove something that
                 # isn't present should we just conclude job done or is that an
                 # error? Nova returns ok if you try to add a group twice.
@@ -358,7 +350,8 @@ class ComputeCloudMixin(_normalize.Normalizer):
             # and the to_munch call.
             self._normalize_server(server._to_munch())
             for server in self.compute.servers(
-                all_projects=all_projects, **filters)]
+                all_projects=all_projects, allow_unknown_params=True,
+                **filters)]
         return [
             self._expand_server(server, detailed, bare)
             for server in servers
@@ -370,10 +363,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
         :returns: A list of server group dicts.
 
         """
-        data = proxy._json_response(
-            self.compute.get('/os-server-groups'),
-            error_message="Error fetching server group list")
-        return self._get_and_munchify('server_groups', data)
+        return list(self.compute.server_groups())
 
     def get_compute_limits(self, name_or_id=None):
         """ Get compute limits for a project
@@ -516,10 +506,12 @@ class ComputeCloudMixin(_normalize.Normalizer):
             return ""
 
     def _get_server_console_output(self, server_id, length=None):
-        data = proxy._json_response(self.compute.post(
-            '/servers/{server_id}/action'.format(server_id=server_id),
-            json={'os-getConsoleOutput': {'length': length}}))
-        return self._get_and_munchify('output', data)
+        output = self.compute.get_server_console_output(
+            server=server_id,
+            length=length
+        )
+        if 'output' in output:
+            return output['output']
 
     def get_server(
             self, name_or_id=None, filters=None, detailed=False, bare=False,
@@ -620,13 +612,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
         }
         if public_key:
             keypair['public_key'] = public_key
-        data = proxy._json_response(
-            self.compute.post(
-                '/os-keypairs',
-                json={'keypair': keypair}),
-            error_message="Unable to create keypair {name}".format(name=name))
-        return self._normalize_keypair(
-            self._get_and_munchify('keypair', data))
+        return self.compute.create_keypair(**keypair)
 
     def delete_keypair(self, name):
         """Delete a keypair.
@@ -638,9 +624,8 @@ class ComputeCloudMixin(_normalize.Normalizer):
         :raises: OpenStackCloudException on operation error.
         """
         try:
-            proxy._json_response(self.compute.delete(
-                '/os-keypairs/{name}'.format(name=name)))
-        except exc.OpenStackCloudURINotFound:
+            self.compute.delete_keypair(name, ignore_missing=False)
+        except exceptions.ResourceNotFound:
             self.log.debug("Keypair %s not found for deleting", name)
             return False
         return True
@@ -673,40 +658,9 @@ class ComputeCloudMixin(_normalize.Normalizer):
                     "Server {server} could not be found and therefore"
                     " could not be snapshotted.".format(server=server))
             server = server_obj
-        response = proxy._json_response(
-            self.compute.post(
-                '/servers/{server_id}/action'.format(server_id=server['id']),
-                json={
-                    "createImage": {
-                        "name": name,
-                        "metadata": metadata,
-                    }
-                }))
-        # You won't believe it - wait, who am I kidding - of course you will!
-        # Nova returns the URL of the image created in the Location
-        # header of the response. (what?) But, even better, the URL it responds
-        # with has a very good chance of being wrong (it is built from
-        # nova.conf values that point to internal API servers in any cloud
-        # large enough to have both public and internal endpoints.
-        # However, nobody has ever noticed this because novaclient doesn't
-        # actually use that URL - it extracts the id from the end of
-        # the url, then returns the id. This leads us to question:
-        #   a) why Nova is going to return a value in a header
-        #   b) why it's going to return data that probably broken
-        #   c) indeed the very nature of the fabric of reality
-        # Although it fills us with existential dread, we have no choice but
-        # to follow suit like a lemming being forced over a cliff by evil
-        # producers from Disney.
-        # TODO(mordred) Update this to consume json microversion when it is
-        #               available.
-        #               blueprint:remove-create-image-location-header-response
-        image_id = response.headers['Location'].rsplit('/', 1)[1]
-        self.list_images.invalidate(self)
-        image = self.get_image(image_id)
-
-        if not wait:
-            return image
-        return self.wait_for_image(image, timeout=timeout)
+        image = self.compute.create_server_image(
+            server, name=name, metadata=metadata, wait=wait, timeout=timeout)
+        return image
 
     def get_server_id(self, name_or_id):
         server = self.get_server(name_or_id, bare=True)
@@ -808,7 +762,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
                                      needed (defaults to True)
         :param network: (optional) Network dict or name or ID to attach the
                         server to.  Mutually exclusive with the nics parameter.
-                        Can also be be a list of network names or IDs or
+                        Can also be a list of network names or IDs or
                         network dicts.
         :param boot_from_volume: Whether to boot from volume. 'boot_volume'
                                  implies True, but boot_from_volume=True with
@@ -836,6 +790,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             raise TypeError(
                 "create_server() requires either 'image' or 'boot_volume'")
 
+        microversion = None
         server_json = {'server': kwargs}
 
         # TODO(mordred) Add support for description starting in 2.19
@@ -896,7 +851,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
                         'Network {network} is not a valid network in'
                         ' {cloud}:{region}'.format(
                             network=network,
-                            cloud=self.name, region=self.config.region_name))
+                            cloud=self.name, region=self._compute_region))
                 nics.append({'net-id': network_obj['id']})
 
             kwargs['nics'] = nics
@@ -929,11 +884,14 @@ class ComputeCloudMixin(_normalize.Normalizer):
                         " may be given")
                 if fixed_ip:
                     net['fixed_ip'] = fixed_ip
-            # TODO(mordred) Add support for tag if server supports microversion
-            # 2.32-2.36 or >= 2.42
             for key in ('port', 'port-id'):
                 if key in nic:
                     net['port'] = nic.pop(key)
+            # A tag supported only in server microversion 2.32-2.36 or >= 2.42
+            # Bumping the version to 2.42 to support the 'tag' implementation
+            if 'tag' in nic:
+                microversion = utils.pick_microversion(self.compute, '2.42')
+                net['tag'] = nic.pop('tag')
             if nic:
                 raise exc.OpenStackCloudException(
                     "Additional unsupported keys given for server network"
@@ -973,7 +931,8 @@ class ComputeCloudMixin(_normalize.Normalizer):
             endpoint = '/os-volumes_boot'
         with _utils.shade_exceptions("Error in creating instance"):
             data = proxy._json_response(
-                self.compute.post(endpoint, json=server_json))
+                self.compute.post(endpoint, json=server_json,
+                                  microversion=microversion))
             server = self._get_and_munchify('server', data)
             admin_pass = server.get('adminPass') or kwargs.get('admin_pass')
             if not wait:
@@ -1025,7 +984,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
                     'Volume {boot_volume} is not a valid volume'
                     ' in {cloud}:{region}'.format(
                         boot_volume=boot_volume,
-                        cloud=self.name, region=self.config.region_name))
+                        cloud=self.name, region=self._compute_region))
             block_mapping = {
                 'boot_index': '0',
                 'delete_on_termination': terminate_volume,
@@ -1046,7 +1005,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
                     'Image {image} is not a valid image in'
                     ' {cloud}:{region}'.format(
                         image=image,
-                        cloud=self.name, region=self.config.region_name))
+                        cloud=self.name, region=self._compute_region))
 
             block_mapping = {
                 'boot_index': '0',
@@ -1076,7 +1035,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
                     'Volume {volume} is not a valid volume'
                     ' in {cloud}:{region}'.format(
                         volume=volume,
-                        cloud=self.name, region=self.config.region_name))
+                        cloud=self.name, region=self._compute_region))
             block_mapping = {
                 'boot_index': '-1',
                 'delete_on_termination': False,
@@ -1095,6 +1054,9 @@ class ComputeCloudMixin(_normalize.Normalizer):
         """
         Wait for a server to reach ACTIVE status.
         """
+        # server = self.compute.wait_for_server(
+        #     server=server, interval=self._SERVER_AGE or 2, wait=timeout
+        # )
         server_id = server['id']
         timeout_message = "Timeout waiting for the server to come up."
         start_time = time.time()
@@ -1233,11 +1195,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             raise exc.OpenStackCloudException(
                 'Invalid Server {server}'.format(server=name_or_id))
 
-        proxy._json_response(
-            self.compute.post(
-                '/servers/{server_id}/metadata'.format(server_id=server['id']),
-                json={'metadata': metadata}),
-            error_message='Error updating server metadata')
+        self.compute.set_server_metadata(server=server.id, **metadata)
 
     def delete_server_metadata(self, name_or_id, metadata_keys):
         """Delete metadata from a server instance.
@@ -1254,15 +1212,8 @@ class ComputeCloudMixin(_normalize.Normalizer):
             raise exc.OpenStackCloudException(
                 'Invalid Server {server}'.format(server=name_or_id))
 
-        for key in metadata_keys:
-            error_message = 'Error deleting metadata {key} on {server}'.format(
-                key=key, server=name_or_id)
-            proxy._json_response(
-                self.compute.delete(
-                    '/servers/{server_id}/metadata/{key}'.format(
-                        server_id=server['id'],
-                        key=key)),
-                error_message=error_message)
+        self.compute.delete_server_metadata(server=server.id,
+                                            keys=metadata_keys)
 
     def delete_server(
             self, name_or_id, wait=False, timeout=180, delete_ips=False,
@@ -1405,7 +1356,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             self._get_and_munchify('server', data))
         return self._expand_server(server, bare=bare, detailed=detailed)
 
-    def create_server_group(self, name, policies):
+    def create_server_group(self, name, policies=[], policy=None):
         """Create a new server group.
 
         :param name: Name of the server group being created
@@ -1415,16 +1366,16 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         :raises: OpenStackCloudException on operation error.
         """
-        data = proxy._json_response(
-            self.compute.post(
-                '/os-server-groups',
-                json={
-                    'server_group': {
-                        'name': name,
-                        'policies': policies}}),
-            error_message="Unable to create server group {name}".format(
-                name=name))
-        return self._get_and_munchify('server_group', data)
+        sg_attrs = {
+            'name': name
+        }
+        if policies:
+            sg_attrs['policies'] = policies
+        if policy:
+            sg_attrs['policy'] = policy
+        return self.compute.create_server_group(
+            **sg_attrs
+        )
 
     def delete_server_group(self, name_or_id):
         """Delete a server group.
@@ -1441,12 +1392,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
                            name_or_id)
             return False
 
-        proxy._json_response(
-            self.compute.delete(
-                '/os-server-groups/{id}'.format(id=server_group['id'])),
-            error_message="Error deleting server group {name}".format(
-                name=name_or_id))
-
+        self.compute.delete_server_group(server_group, ignore_missing=False)
         return True
 
     def create_flavor(self, name, ram, vcpus, disk, flavorid="auto",
@@ -1594,16 +1540,16 @@ class ComputeCloudMixin(_normalize.Normalizer):
         return _utils.normalize_flavor_accesses(
             self._get_and_munchify('flavor_access', data))
 
-    def list_hypervisors(self):
+    def list_hypervisors(self, filters={}):
         """List all hypervisors
 
         :returns: A list of hypervisor ``munch.Munch``.
         """
 
-        data = proxy._json_response(
-            self.compute.get('/os-hypervisors/detail'),
-            error_message="Error fetching hypervisor list")
-        return self._get_and_munchify('hypervisors', data)
+        return list(self.compute.hypervisors(
+            details=True,
+            allow_unknown_params=True,
+            **filters))
 
     def search_aggregates(self, name_or_id=None, filters=None):
         """Seach host aggregates.
@@ -1619,16 +1565,13 @@ class ComputeCloudMixin(_normalize.Normalizer):
         aggregates = self.list_aggregates()
         return _utils._filter_list(aggregates, name_or_id, filters)
 
-    def list_aggregates(self):
+    def list_aggregates(self, filters={}):
         """List all available host aggregates.
 
         :returns: A list of aggregate dicts.
 
         """
-        data = proxy._json_response(
-            self.compute.get('/os-aggregates'),
-            error_message="Error fetching aggregate list")
-        return self._get_and_munchify('aggregates', data)
+        return self.compute.aggregates(allow_unknown_params=True, **filters)
 
     def get_aggregate(self, name_or_id, filters=None):
         """Get an aggregate by name or ID.
@@ -1661,16 +1604,10 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         :raises: OpenStackCloudException on operation error.
         """
-        data = proxy._json_response(
-            self.compute.post(
-                '/os-aggregates',
-                json={'aggregate': {
-                    'name': name,
-                    'availability_zone': availability_zone
-                }}),
-            error_message="Unable to create host aggregate {name}".format(
-                name=name))
-        return self._get_and_munchify('aggregate', data)
+        return self.compute.create_aggregate(
+            name=name,
+            availability_zone=availability_zone
+        )
 
     @_utils.valid_kwargs('name', 'availability_zone')
     def update_aggregate(self, name_or_id, **kwargs):
@@ -1685,17 +1622,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
         :raises: OpenStackCloudException on operation error.
         """
         aggregate = self.get_aggregate(name_or_id)
-        if not aggregate:
-            raise exc.OpenStackCloudException(
-                "Host aggregate %s not found." % name_or_id)
-
-        data = proxy._json_response(
-            self.compute.put(
-                '/os-aggregates/{id}'.format(id=aggregate['id']),
-                json={'aggregate': kwargs}),
-            error_message="Error updating aggregate {name}".format(
-                name=name_or_id))
-        return self._get_and_munchify('aggregate', data)
+        return self.compute.update_aggregate(aggregate, **kwargs)
 
     def delete_aggregate(self, name_or_id):
         """Delete a host aggregate.
@@ -1706,19 +1633,13 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         :raises: OpenStackCloudException on operation error.
         """
-        aggregate = self.get_aggregate(name_or_id)
-        if not aggregate:
+        try:
+            self.compute.delete_aggregate(name_or_id, ignore_missing=False)
+            return True
+        except exceptions.ResourceNotFound:
             self.log.debug("Aggregate %s not found for deleting", name_or_id)
             return False
 
-        return proxy._json_response(
-            self.compute.delete(
-                '/os-aggregates/{id}'.format(id=aggregate['id'])),
-            error_message="Error deleting aggregate {name}".format(
-                name=name_or_id))
-
-        return True
-
     def set_aggregate_metadata(self, name_or_id, metadata):
         """Set aggregate metadata, replacing the existing metadata.
 
@@ -1735,15 +1656,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             raise exc.OpenStackCloudException(
                 "Host aggregate %s not found." % name_or_id)
 
-        err_msg = "Unable to set metadata for host aggregate {name}".format(
-            name=name_or_id)
-
-        data = proxy._json_response(
-            self.compute.post(
-                '/os-aggregates/{id}/action'.format(id=aggregate['id']),
-                json={'set_metadata': {'metadata': metadata}}),
-            error_message=err_msg)
-        return self._get_and_munchify('aggregate', data)
+        return self.compute.set_aggregate_metadata(aggregate, metadata)
 
     def add_host_to_aggregate(self, name_or_id, host_name):
         """Add a host to an aggregate.
@@ -1758,14 +1671,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             raise exc.OpenStackCloudException(
                 "Host aggregate %s not found." % name_or_id)
 
-        err_msg = "Unable to add host {host} to aggregate {name}".format(
-            host=host_name, name=name_or_id)
-
-        return proxy._json_response(
-            self.compute.post(
-                '/os-aggregates/{id}/action'.format(id=aggregate['id']),
-                json={'add_host': {'host': host_name}}),
-            error_message=err_msg)
+        return self.compute.add_host_to_aggregate(aggregate, host_name)
 
     def remove_host_from_aggregate(self, name_or_id, host_name):
         """Remove a host from an aggregate.
@@ -1780,14 +1686,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             raise exc.OpenStackCloudException(
                 "Host aggregate %s not found." % name_or_id)
 
-        err_msg = "Unable to remove host {host} to aggregate {name}".format(
-            host=host_name, name=name_or_id)
-
-        return proxy._json_response(
-            self.compute.post(
-                '/os-aggregates/{id}/action'.format(id=aggregate['id']),
-                json={'remove_host': {'host': host_name}}),
-            error_message=err_msg)
+        return self.compute.remove_host_from_aggregate(aggregate, host_name)
 
     def set_compute_quotas(self, name_or_id, **kwargs):
         """ Set a quota in a project
