@@ -9,18 +9,13 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
-import hashlib
-
-from openstack import _log
 from openstack import exceptions
+from openstack.image import _download
 from openstack import resource
 from openstack import utils
 
-_logger = _log.setup_logging('openstack')
 
-
-class Image(resource.Resource, resource.TagMixin):
+class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
     resources_key = 'images'
     base_path = '/images'
 
@@ -33,13 +28,18 @@ class Image(resource.Resource, resource.TagMixin):
     commit_method = 'PATCH'
     commit_jsonpatch = True
 
+    # Store all unknown attributes under 'properties' in the object.
+    # Remotely they would be still in the resource root
+    _store_unknown_attrs_as_properties = True
+
     _query_mapping = resource.QueryParameters(
         "name", "visibility",
         "member_status", "owner",
-        "status", "size_min",
-        "size_max", "sort_key",
-        "sort_dir", "sort", "tag",
-        "created_at", "updated_at")
+        "status", "size_min", "size_max",
+        "protected", "is_hidden",
+        "sort_key", "sort_dir", "sort", "tag",
+        "created_at", "updated_at",
+        is_hidden="os_hidden")
 
     # NOTE: Do not add "self" support here. If you've used Python before,
     # you know that self, while not being a reserved word, has special
@@ -69,9 +69,18 @@ class Image(resource.Resource, resource.TagMixin):
     #: disk image. Virtual appliance vendors have different formats
     #: for laying out the information contained in a VM disk image.
     disk_format = resource.Body('disk_format')
+    #: This field controls whether an image is displayed in the default
+    #: image-list response
+    is_hidden = resource.Body('os_hidden', type=bool)
     #: Defines whether the image can be deleted.
     #: *Type: bool*
     is_protected = resource.Body('protected', type=bool)
+    #: The algorithm used to compute a secure hash of the image data
+    #: for this image
+    hash_algo = resource.Body('os_hash_algo')
+    #: The hexdigest of the secure hash of the image data computed using
+    #: the algorithm whose name is the value of the os_hash_algo property.
+    hash_value = resource.Body('os_hash_value')
     #: The minimum disk size in GB that is required to boot the image.
     min_disk = resource.Body('min_disk')
     #: The minimum amount of RAM in MB that is required to boot the image.
@@ -95,8 +104,6 @@ class Image(resource.Resource, resource.TagMixin):
     store = resource.Body('store')
     #: The image status.
     status = resource.Body('status')
-    #: Tags, if any, that are associated with the image.
-    tags = resource.Body('tags')
     #: The date and time when the image was updated.
     updated_at = resource.Body('updated_at')
     #: The virtual size of the image.
@@ -125,7 +132,7 @@ class Image(resource.Resource, resource.TagMixin):
     architecture = resource.Body("architecture")
     #: The hypervisor type. Note that qemu is used for both QEMU and
     #: KVM hypervisor types.
-    hypervisor_type = resource.Body("hypervisor-type")
+    hypervisor_type = resource.Body("hypervisor_type")
     #: Optional property allows created servers to have a different bandwidth
     #: cap than that defined in the network they are attached to.
     instance_type_rxtx_factor = resource.Body(
@@ -147,6 +154,8 @@ class Image(resource.Resource, resource.TagMixin):
     #: Secure Boot first examines software such as firmware and OS by
     #: their signature and only allows them to run if the signatures are valid.
     needs_secure_boot = resource.Body('os_secure_boot')
+    #: Time for graceful shutdown
+    os_shutdown_timeout = resource.Body('os_shutdown_timeout', type=int)
     #: The ID of image stored in the Image service that should be used as
     #: the ramdisk when booting an AMI-style image.
     ramdisk_id = resource.Body('ramdisk_id')
@@ -162,6 +171,12 @@ class Image(resource.Resource, resource.TagMixin):
     #: Specifies the type of disk controller to attach disk devices to.
     #: One of scsi, virtio, uml, xen, ide, or usb.
     hw_disk_bus = resource.Body('hw_disk_bus')
+    #: Used to pin the virtual CPUs (vCPUs) of instances to the
+    #: host's physical CPU cores (pCPUs).
+    hw_cpu_policy = resource.Body('hw_cpu_policy')
+    #: Defines how hardware CPU threads in a simultaneous
+    #: multithreading-based (SMT) architecture be used.
+    hw_cpu_thread_policy = resource.Body('hw_cpu_thread_policy')
     #: Adds a random-number generator device to the image's instances.
     hw_rng_model = resource.Body('hw_rng_model')
     #: For libvirt: Enables booting an ARM system using the specified
@@ -202,7 +217,7 @@ class Image(resource.Resource, resource.TagMixin):
     vmware_ostype = resource.Body('vmware_ostype')
     #: If true, the root partition on the disk is automatically resized
     #: before the instance boots.
-    has_auto_disk_config = resource.Body('auto_disk_config', type=bool)
+    has_auto_disk_config = resource.Body('auto_disk_config')
     #: The operating system installed on the image.
     os_type = resource.Body('os_type')
     #: The operating system admin username.
@@ -211,6 +226,8 @@ class Image(resource.Resource, resource.TagMixin):
     hw_qemu_guest_agent = resource.Body('hw_qemu_guest_agent', type=bool)
     #: If true, require quiesce on snapshot via QEMU guest agent.
     os_require_quiesce = resource.Body('os_require_quiesce', type=bool)
+    #: The URL for the schema describing a virtual machine image.
+    schema = resource.Body('schema')
 
     def _action(self, session, action):
         """Call an action on an image ID."""
@@ -235,47 +252,37 @@ class Image(resource.Resource, resource.TagMixin):
     def upload(self, session):
         """Upload data into an existing image"""
         url = utils.urljoin(self.base_path, self.id, 'file')
-        session.put(url, data=self.data,
-                    headers={"Content-Type": "application/octet-stream",
-                             "Accept": ""})
+        return session.put(url, data=self.data,
+                           headers={"Content-Type": "application/octet-stream",
+                                    "Accept": ""})
 
-    def download(self, session, stream=False):
-        """Download the data contained in an image"""
-        # TODO(briancurtin): This method should probably offload the get
-        # operation into another thread or something of that nature.
-        url = utils.urljoin(self.base_path, self.id, 'file')
-        resp = session.get(url, stream=stream)
+    def stage(self, session):
+        """Stage binary image data into an existing image"""
+        url = utils.urljoin(self.base_path, self.id, 'stage')
+        response = session.put(
+            url, data=self.data,
+            headers={"Content-Type": "application/octet-stream",
+                     "Accept": ""})
+        self._translate_response(response, has_body=False)
+        return self
 
-        # See the following bug report for details on why the checksum
-        # code may sometimes depend on a second GET call.
-        # https://storyboard.openstack.org/#!/story/1619675
-        checksum = resp.headers.get("Content-MD5")
-
-        if checksum is None:
-            # If we don't receive the Content-MD5 header with the download,
-            # make an additional call to get the image details and look at
-            # the checksum attribute.
-            details = self.fetch(session)
-            checksum = details.checksum
-
-        # if we are returning the repsonse object, ensure that it
-        # has the content-md5 header so that the caller doesn't
-        # need to jump through the same hoops through which we
-        # just jumped.
-        if stream:
-            resp.headers['content-md5'] = checksum
-            return resp
-
-        if checksum is not None:
-            digest = hashlib.md5(resp.content).hexdigest()
-            if digest != checksum:
-                raise exceptions.InvalidResponse(
-                    "checksum mismatch: %s != %s" % (checksum, digest))
-        else:
-            _logger.warn(
-                "Unable to verify the integrity of image %s" % (self.id))
-
-        return resp.content
+    def import_image(self, session, method='glance-direct', uri=None,
+                     store=None):
+        """Import Image via interoperable image import process"""
+        url = utils.urljoin(self.base_path, self.id, 'import')
+        json = {'method': {'name': method}}
+        if uri:
+            if method == 'web-download':
+                json['method']['uri'] = uri
+            elif method == 'glance-direct':
+                json['method']['uri'] = uri
+            else:
+                raise exceptions.InvalidRequest('URI is only supported with '
+                                                'method: "web-download"')
+        headers = {}
+        if store is not None:
+            headers = {'X-Image-Meta-Store': store.id}
+        session.post(url, json=json, headers=headers)
 
     def _prepare_request(self, requires_id=None, prepend_key=False,
                          patch=False, base_path=None):
@@ -291,3 +298,25 @@ class Image(resource.Resource, resource.TagMixin):
             request.headers.update(headers)
 
         return request
+
+    @classmethod
+    def find(cls, session, name_or_id, ignore_missing=True, **params):
+        # Do a regular search first (ignoring missing)
+        result = super(Image, cls).find(session, name_or_id, True,
+                                        **params)
+
+        if result:
+            return result
+        else:
+            # Search also in hidden images
+            params['is_hidden'] = True
+            data = cls.list(session, **params)
+
+            result = cls._get_one_match(name_or_id, data)
+            if result is not None:
+                return result
+
+        if ignore_missing:
+            return None
+        raise exceptions.ResourceNotFound(
+            "No %s found for %s" % (cls.__name__, name_or_id))
