@@ -32,6 +32,7 @@ and then returned to the caller.
 """
 
 import collections
+import inspect
 import itertools
 
 import jsonpatch
@@ -88,7 +89,7 @@ class _BaseComponent(object):
     # The class to be used for mappings
     _map_cls = dict
 
-    def __init__(self, name, type=None, default=None, alias=None,
+    def __init__(self, name, type=None, default=None, alias=None, aka=None,
                  alternate_id=False, list_type=None, coerce_to_default=False,
                  **kwargs):
         """A typed descriptor for a component that makes up a Resource
@@ -101,6 +102,7 @@ class _BaseComponent(object):
             component to a string, __set__ will fail, for example.
         :param default: Typically None, but any other default can be set.
         :param alias: If set, alternative attribute on object to return.
+        :param aka: If set, additional name attribute would be available under.
         :param alternate_id:
             When `True`, this property is known internally as a value that
             can be sent with requests that require an ID but when `id` is
@@ -121,6 +123,7 @@ class _BaseComponent(object):
         else:
             self.default = default
         self.alias = alias
+        self.aka = aka
         self.alternate_id = alternate_id
         self.list_type = list_type
         self.coerce_to_default = coerce_to_default
@@ -287,29 +290,41 @@ class QueryParameters(object):
         self._mapping.update({name: name for name in names})
         self._mapping.update(mappings)
 
-    def _validate(self, query, base_path=None):
+    def _validate(self, query, base_path=None, allow_unknown_params=False):
         """Check that supplied query keys match known query mappings
 
         :param dict query: Collection of key-value pairs where each key is the
-                           client-side parameter name or server side name.
+            client-side parameter name or server side name.
         :param base_path: Formatted python string of the base url path for
-                          the resource.
+            the resource.
+        : param allow_unknown_params: Exclude query params not known by the
+            resource.
+
+        :returns: Filtered collection of the supported QueryParameters
         """
         expected_params = list(self._mapping.keys())
         expected_params.extend(
-            value['name'] if isinstance(value, dict) else value
-            for value in self._mapping.values())
+            value.get('name', key) if isinstance(value, dict) else value
+            for key, value in self._mapping.items())
 
         if base_path:
             expected_params += utils.get_string_format_keys(base_path)
 
         invalid_keys = set(query.keys()) - set(expected_params)
-        if invalid_keys:
-            raise exceptions.InvalidResourceQuery(
-                message="Invalid query params: %s" % ",".join(invalid_keys),
-                extra_data=invalid_keys)
+        if not invalid_keys:
+            return query
+        else:
+            if not allow_unknown_params:
+                raise exceptions.InvalidResourceQuery(
+                    message="Invalid query params: %s" %
+                    ",".join(invalid_keys),
+                    extra_data=invalid_keys)
+            else:
+                known_keys = set(query.keys()).intersection(
+                    set(expected_params))
+                return {k: query[k] for k in known_keys}
 
-    def _transpose(self, query):
+    def _transpose(self, query, resource_type):
         """Transpose the keys in query based on the mapping
 
         If a query is supplied with its server side name, we will still use
@@ -318,16 +333,25 @@ class QueryParameters(object):
         :param dict query: Collection of key-value pairs where each key is the
                            client-side parameter name to be transposed to its
                            server side name.
+        :param resource_type: Class of a resource.
         """
         result = {}
         for client_side, server_side in self._mapping.items():
             if isinstance(server_side, dict):
-                name = server_side['name']
+                name = server_side.get('name', client_side)
                 type_ = server_side.get('type')
             else:
                 name = server_side
                 type_ = None
 
+            # NOTE(dtantsur): a small hack to be compatible with both
+            # single-argument (like int) and double-argument type functions.
+            try:
+                provide_resource_type = (
+                    len(inspect.getargspec(type_).args) > 1)
+            except TypeError:
+                provide_resource_type = False
+
             if client_side in query:
                 value = query[client_side]
             elif name in query:
@@ -336,7 +360,10 @@ class QueryParameters(object):
                 continue
 
             if type_ is not None:
-                result[name] = type_(value)
+                if provide_resource_type:
+                    result[name] = type_(value, resource_type)
+                else:
+                    result[name] = type_(value)
             else:
                 result[name] = value
         return result
@@ -428,6 +455,9 @@ class Resource(dict):
     _delete_response_class = None
     _store_unknown_attrs_as_properties = False
 
+    # Placeholder for aliases as dict of {__alias__:__original}
+    _attr_aliases = {}
+
     def __init__(self, _synchronized=False, connection=None, **attrs):
         """The base resource
 
@@ -481,6 +511,11 @@ class Resource(dict):
 
         self._update_location()
 
+        for attr, component in self._attributes_iterator():
+            if component.aka:
+                # Register alias for the attribute (local name)
+                self._attr_aliases[component.aka] = attr
+
         # TODO(mordred) This is terrible, but is a hack at the moment to ensure
         # json.dumps works. The json library does basically if not obj: and
         # obj.items() ... but I think the if not obj: is short-circuiting down
@@ -488,6 +523,18 @@ class Resource(dict):
         # always False even if we override __len__ or __bool__.
         dict.update(self, self.to_dict())
 
+    @classmethod
+    def _attributes_iterator(cls, components=tuple([Body, Header])):
+        """Iterator over all Resource attributes
+        """
+        # isinstance stricly requires this to be a tuple
+        # Since we're looking at class definitions we need to include
+        # subclasses, so check the whole MRO.
+        for klass in cls.__mro__:
+            for attr, component in klass.__dict__.items():
+                if isinstance(component, components):
+                    yield attr, component
+
     def __repr__(self):
         pairs = [
             "%s=%s" % (k, v if v is not None else 'None')
@@ -529,7 +576,14 @@ class Resource(dict):
                 except KeyError:
                     return None
         else:
-            return object.__getattribute__(self, name)
+            try:
+                return object.__getattribute__(self, name)
+            except AttributeError as e:
+                if name in self._attr_aliases:
+                    # Hmm - not found. But hey, the alias exists...
+                    return object.__getattribute__(
+                        self, self._attr_aliases[name])
+                raise e
 
     def __getitem__(self, name):
         """Provide dictionary access for elements of the data model."""
@@ -537,6 +591,10 @@ class Resource(dict):
         # behaves like its wrapped content. If we get it on the class,
         # it returns the BaseComponent itself, not the results of __get__.
         real_item = getattr(self.__class__, name, None)
+        if not real_item and name in self._attr_aliases:
+            # Not found? But we know an alias exists.
+            name = self._attr_aliases[name]
+            real_item = getattr(self.__class__, name, None)
         if isinstance(real_item, _BaseComponent):
             return getattr(self, name)
         raise KeyError(name)
@@ -557,6 +615,23 @@ class Resource(dict):
                     cls=self.__class__.__name__,
                     name=name))
 
+    def _attributes(self, remote_names=False, components=None,
+                    include_aliases=True):
+        """Generate list of supported attributes
+        """
+        attributes = []
+
+        if not components:
+            components = tuple([Body, Header, Computed, URI])
+
+        for attr, component in self._attributes_iterator(components):
+            key = attr if not remote_names else component.name
+            attributes.append(key)
+            if include_aliases and component.aka:
+                attributes.append(component.aka)
+
+        return attributes
+
     def keys(self):
         # NOTE(mordred) In python2, dict.keys returns a list. In python3 it
         # returns a dict_keys view. For 2, we can return a list from the
@@ -564,15 +639,25 @@ class Resource(dict):
         # It won't strictly speaking be an actual dict_keys, so it's possible
         # we may want to get more clever, but for now let's see how far this
         # will take us.
-        underlying_keys = itertools.chain(
-            self._body.attributes.keys(),
-            self._header.attributes.keys(),
-            self._uri.attributes.keys(),
-            self._computed.attributes.keys())
-        if six.PY2:
-            return list(underlying_keys)
-        else:
-            return underlying_keys
+        # NOTE(gtema) For now let's return list of 'public' attributes and not
+        # remotes or "unknown"
+        return self._attributes()
+
+    def items(self):
+        # This method is critically required for Ansible "jsonify"
+        # NOTE(gtema) For some reason when running from SDK itself the native
+        # implementation of the method is absolutely sifficient, when called
+        # from Ansible - the values are often empty. Even integrating all
+        # Ansible internal methods did not help to find the root cause. Another
+        # fact is that under Py2 everything is fine, while under Py3 it fails.
+        # There is currently no direct test for Ansible-SDK issue. It is tested
+        # implicitely in the keypair role for ansible module, where an assert
+        # verifies presence of attributes.
+        res = []
+        for attr in self._attributes():
+            # Append key, value tuple to result list
+            res.append((attr, self[attr]))
+        return res
 
     def _update(self, **attrs):
         """Given attributes, update them on this instance
@@ -724,16 +809,12 @@ class Resource(dict):
         """Return a dict of attributes of a given component on the class"""
         mapping = component._map_cls()
         ret = component._map_cls()
-        # Since we're looking at class definitions we need to include
-        # subclasses, so check the whole MRO.
-        for klass in cls.__mro__:
-            for key, value in klass.__dict__.items():
-                if isinstance(value, component):
-                    # Make sure base classes don't end up overwriting
-                    # mappings we've found previously in subclasses.
-                    if key not in mapping:
-                        # Make it this way first, to get MRO stuff correct.
-                        mapping[key] = value.name
+        for key, value in cls._attributes_iterator(component):
+            # Make sure base classes don't end up overwriting
+            # mappings we've found previously in subclasses.
+            if key not in mapping:
+                # Make it this way first, to get MRO stuff correct.
+                mapping[key] = value.name
         for k, v in mapping.items():
             ret[v] = k
         return ret
@@ -876,38 +957,35 @@ class Resource(dict):
         # but is slightly different in that we're looking at an instance
         # and we're mapping names on this class to their actual stored
         # values.
-        # Since we're looking at class definitions we need to include
-        # subclasses, so check the whole MRO.
-        for klass in self.__class__.__mro__:
-            for attr, component in klass.__dict__.items():
-                if isinstance(component, components):
-                    if original_names:
-                        key = component.name
+        for attr, component in self._attributes_iterator(components):
+            if original_names:
+                key = component.name
+            else:
+                key = attr
+            for key in filter(None, (key, component.aka)):
+                # Make sure base classes don't end up overwriting
+                # mappings we've found previously in subclasses.
+                if key not in mapping:
+                    value = getattr(self, attr, None)
+                    if ignore_none and value is None:
+                        continue
+                    if isinstance(value, Resource):
+                        mapping[key] = value.to_dict(_to_munch=_to_munch)
+                    elif isinstance(value, dict) and _to_munch:
+                        mapping[key] = munch.Munch(value)
+                    elif value and isinstance(value, list):
+                        converted = []
+                        for raw in value:
+                            if isinstance(raw, Resource):
+                                converted.append(
+                                    raw.to_dict(_to_munch=_to_munch))
+                            elif isinstance(raw, dict) and _to_munch:
+                                converted.append(munch.Munch(raw))
+                            else:
+                                converted.append(raw)
+                        mapping[key] = converted
                     else:
-                        key = attr
-                    # Make sure base classes don't end up overwriting
-                    # mappings we've found previously in subclasses.
-                    if key not in mapping:
-                        value = getattr(self, attr, None)
-                        if ignore_none and value is None:
-                            continue
-                        if isinstance(value, Resource):
-                            mapping[key] = value.to_dict(_to_munch=_to_munch)
-                        elif isinstance(value, dict) and _to_munch:
-                            mapping[key] = munch.Munch(value)
-                        elif value and isinstance(value, list):
-                            converted = []
-                            for raw in value:
-                                if isinstance(raw, Resource):
-                                    converted.append(
-                                        raw.to_dict(_to_munch=_to_munch))
-                                elif isinstance(raw, dict) and _to_munch:
-                                    converted.append(munch.Munch(raw))
-                                else:
-                                    converted.append(raw)
-                            mapping[key] = converted
-                        else:
-                            mapping[key] = value
+                        mapping[key] = value
 
         return mapping
     # Compatibility with the munch.Munch.toDict method
@@ -975,7 +1053,7 @@ class Resource(dict):
         return body
 
     def _prepare_request(self, requires_id=None, prepend_key=False,
-                         patch=False, base_path=None):
+                         patch=False, base_path=None, params=None):
         """Prepare a request to be sent to the server
 
         Create operations don't require an ID, but all others do,
@@ -1013,6 +1091,10 @@ class Resource(dict):
 
             uri = utils.urljoin(uri, self.id)
 
+        if params:
+            query_params = six.moves.urllib.parse.urlencode(params)
+            uri += '?' + query_params
+
         return _Request(uri, body, headers)
 
     def _translate_response(self, response, has_body=None, error_message=None):
@@ -1053,6 +1135,7 @@ class Resource(dict):
         self._header.attributes.update(headers)
         self._header.clean()
         self._update_location()
+        dict.update(self, self.to_dict())
 
     @classmethod
     def _get_session(cls, session):
@@ -1151,7 +1234,7 @@ class Resource(dict):
 
         return actual
 
-    def create(self, session, prepend_key=True, base_path=None):
+    def create(self, session, prepend_key=True, base_path=None, **params):
         """Create a remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -1162,6 +1245,7 @@ class Resource(dict):
         :param str base_path: Base part of the URI for creating resources, if
                               different from
                               :data:`~openstack.resource.Resource.base_path`.
+        :param dict params: Additional params to pass.
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                  :data:`Resource.allow_create` is not set to ``True``.
@@ -1180,14 +1264,14 @@ class Resource(dict):
                                             base_path=base_path)
             response = session.put(request.url,
                                    json=request.body, headers=request.headers,
-                                   microversion=microversion)
+                                   microversion=microversion, params=params)
         elif self.create_method == 'POST':
             request = self._prepare_request(requires_id=requires_id,
                                             prepend_key=prepend_key,
                                             base_path=base_path)
             response = session.post(request.url,
                                     json=request.body, headers=request.headers,
-                                    microversion=microversion)
+                                    microversion=microversion, params=params)
         else:
             raise exceptions.ResourceFailure(
                 msg="Invalid create method: %s" % self.create_method)
@@ -1229,7 +1313,8 @@ class Resource(dict):
                                         base_path=base_path)
         session = self._get_session(session)
         microversion = self._get_microversion_for(session, 'fetch')
-        response = session.get(request.url, microversion=microversion)
+        response = session.get(request.url, microversion=microversion,
+                               params=params)
         kwargs = {}
         if error_message:
             kwargs['error_message'] = error_message
@@ -1261,7 +1346,6 @@ class Resource(dict):
         session = self._get_session(session)
         microversion = self._get_microversion_for(session, 'fetch')
         response = session.head(request.url,
-                                headers={"Accept": ""},
                                 microversion=microversion)
 
         self.microversion = microversion
@@ -1452,11 +1536,11 @@ class Resource(dict):
 
         return session.delete(
             request.url,
-            headers={"Accept": ""},
             microversion=microversion)
 
     @classmethod
-    def list(cls, session, paginated=True, base_path=None, **params):
+    def list(cls, session, paginated=True, base_path=None,
+             allow_unknown_params=False, **params):
         """This method is a generator which yields resource objects.
 
         This resource object list generator handles pagination and takes query
@@ -1465,14 +1549,17 @@ class Resource(dict):
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
         :param bool paginated: ``True`` if a GET to this resource returns
-                               a paginated series of responses, or ``False``
-                               if a GET returns only one page of data.
-                               **When paginated is False only one
-                               page of data will be returned regardless
-                               of the API's support of pagination.**
+            a paginated series of responses, or ``False``
+            if a GET returns only one page of data.
+            **When paginated is False only one
+            page of data will be returned regardless
+            of the API's support of pagination.**
         :param str base_path: Base part of the URI for listing resources, if
-                              different from
-                              :data:`~openstack.resource.Resource.base_path`.
+            different from :data:`~openstack.resource.Resource.base_path`.
+        :param bool allow_unknown_params: ``True`` to accept, but discard
+            unknown query parameters. This allows getting list of 'filters' and
+            passing everything known to the server. ``False`` will result in
+            validation exception when unknown query parameters are passed.
         :param dict params: These keyword arguments are passed through the
             :meth:`~openstack.resource.QueryParamter._transpose` method
             to find if any of them match expected query parameters to be
@@ -1496,8 +1583,10 @@ class Resource(dict):
 
         if base_path is None:
             base_path = cls.base_path
-        cls._query_mapping._validate(params, base_path=base_path)
-        query_params = cls._query_mapping._transpose(params)
+        params = cls._query_mapping._validate(
+            params, base_path=base_path,
+            allow_unknown_params=allow_unknown_params)
+        query_params = cls._query_mapping._transpose(params, cls)
         uri = base_path % params
 
         limit = query_params.get('limit')
@@ -1568,7 +1657,7 @@ class Resource(dict):
                 links = data.get(pagination_key, {})
                 # keystone might return a dict
                 if isinstance(links, dict):
-                    links = ({k: v} for k, v in six.iteritems(links))
+                    links = ({k: v} for k, v in links.items())
                 for item in links:
                     if item.get('rel') == 'next' and 'href' in item:
                         next_link = item['href']
