@@ -16,6 +16,7 @@ import mock
 from openstack.baremetal.v1 import _common
 from openstack.baremetal.v1 import node
 from openstack import exceptions
+from openstack import resource
 from openstack.tests.unit import base
 
 # NOTE: Sample data from api-ref doc
@@ -50,6 +51,7 @@ FAKE = {
     "maintenance_reason": None,
     "name": "test_node",
     "network_interface": "flat",
+    "owner": "4b7ed919-e4a6-4017-a081-43205c5b0b73",
     "portgroups": [
         {
             "href": "http://127.0.0.1:6385/v1/nodes/<NODE_ID>/portgroups",
@@ -132,6 +134,7 @@ class TestNode(base.TestCase):
         self.assertEqual(FAKE['maintenance_reason'], sot.maintenance_reason)
         self.assertEqual(FAKE['name'], sot.name)
         self.assertEqual(FAKE['network_interface'], sot.network_interface)
+        self.assertEqual(FAKE['owner'], sot.owner)
         self.assertEqual(FAKE['ports'], sot.ports)
         self.assertEqual(FAKE['portgroups'], sot.port_groups)
         self.assertEqual(FAKE['power_state'], sot.power_state)
@@ -764,3 +767,39 @@ class TestNodeTraits(base.TestCase):
             json={'traits': ['CUSTOM_FAKE', 'CUSTOM_REAL', 'CUSTOM_MISSING']},
             headers=mock.ANY, microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+
+@mock.patch.object(resource.Resource, 'patch', autospec=True)
+class TestNodePatch(base.TestCase):
+
+    def setUp(self):
+        super(TestNodePatch, self).setUp()
+        self.node = node.Node(**FAKE)
+        self.session = mock.Mock(spec=adapter.Adapter,
+                                 default_microversion=None)
+        self.session.log = mock.Mock()
+
+    def test_node_patch(self, mock_patch):
+        patch = {'path': 'test'}
+        self.node.patch(self.session, patch=patch)
+        mock_patch.assert_called_once()
+        kwargs = mock_patch.call_args.kwargs
+        self.assertEqual(kwargs['patch'], {'path': 'test'})
+
+    @mock.patch.object(resource.Resource, '_prepare_request', autospec=True)
+    @mock.patch.object(resource.Resource, '_commit', autospec=True)
+    def test_node_patch_reset_interfaces(self, mock__commit, mock_prepreq,
+                                         mock_patch):
+        patch = {'path': 'test'}
+        self.node.patch(self.session, patch=patch, retry_on_conflict=True,
+                        reset_interfaces=True)
+        mock_prepreq.assert_called_once()
+        prepreq_kwargs = mock_prepreq.call_args.kwargs
+        self.assertEqual(prepreq_kwargs['params'],
+                         [('reset_interfaces', True)])
+        mock__commit.assert_called_once()
+        commit_args = mock__commit.call_args.args
+        commit_kwargs = mock__commit.call_args.kwargs
+        self.assertIn('1.45', commit_args)
+        self.assertEqual(commit_kwargs['retry_on_conflict'], True)
+        mock_patch.assert_not_called()
